%% Adaptive coder
clc
clear all

% Read avi file
movObj = mmreader('screen_content_1.avi');

% Convert to 4D matrix with coefficients
vidFrames = read(movObj);

display('Adaptive coding')

% Find height, width and number of frames
sizeOfMatrix = size(vidFrames);
nHeight = sizeOfMatrix(1);
nWidth = sizeOfMatrix(2);
nFrames = sizeOfMatrix(4);

% Set quantization parameter, cannot be set to 0
% Set iFrame frequency (GOP size) and lambda (Lagrange multiplier)
totalq = 32;
iFrameFreq = 6;
lambda = 0.85;
      
% Initialize variables
rFrames = uint8(zeros(nHeight, nWidth,nFrames));
gFrames = uint8(zeros(nHeight, nWidth,nFrames));
bFrames = uint8(zeros(nHeight, nWidth,nFrames));

yFrames = zeros(nHeight, nWidth,nFrames);
uFrames = uint8(zeros(nHeight, nWidth,nFrames));
vFrames = uint8(zeros(nHeight, nWidth,nFrames));

%dpcmYFrames = zeros(nHeight, nWidth,nFrames);  
%recYFrames = zeros(nHeight, nWidth,nFrames);
%codedYFrames = zeros(nHeight, nWidth,nFrames);

yCells = cell(1,nFrames);
rsqYCells = cell(1,nFrames);
origYCells = cell(1,nFrames);
uCells = cell(1,nFrames);
vCells = cell(1,nFrames);

% Split into R, G and B frames
for i = 1:nFrames
    rFrames(:,:,i) = vidFrames(:,:,1,i);
    gFrames(:,:,i) = vidFrames(:,:,2,i);
    bFrames(:,:,i) = vidFrames(:,:,3,i);
    
    % Convert from RGB to YUV
    [yFrames(:,:,i), uFrames(:,:,i), vFrames(:,:,i)] = rgb2yuv(rFrames(:,:,i), gFrames(:,:,i), bFrames(:,:,i));    
end

blockSize = 4;
blocks = zeros(blockSize,blockSize,nWidth*nHeight/(blockSize.^2));
recOrigYFrames = zeros(size(yFrames));
recRSQYFrames = zeros(size(yFrames)); 
recYFrames = zeros(size(yFrames));
dpcmBlocks = zeros(size(blocks));
quantizedOrigBlocks = zeros(size(blocks));
quantizedRSQBlocks = zeros(size(blocks));
iQuantOrigBlocks = zeros(size(blocks));
recOrigBlocks = zeros(size(blocks));
recRSQBlocks = zeros(size(blocks));
tempBlock = zeros(blockSize,blockSize);
tempBlockOrig = zeros(blockSize,blockSize);
rdOrig = zeros(2,length(blocks));
rdRSQ = zeros(2,length(blocks));

psnrVector = zeros(1,nFrames);
rsqPSNRVector = zeros(1,nFrames);
origPSNRVector = zeros(1,nFrames);

totalRateRSQ = 0;
totalRateOrig = 0;
totalRateAdaptive = 0;

% Encoding
for i = 1:nFrames         
% for i = 1:1         
    % Split frames into blocks and put them in a vector
    l = 1;
    j = blockSize;
    k = blockSize;
    for p = blockSize:blockSize:nWidth*nHeight/blockSize                    
        blocks(:,:,l) = yFrames(k-(blockSize-1):k,j-(blockSize-1):j,i);
        k = k + blockSize;
        if mod(p,nHeight) == 0
           j = j+blockSize;
           k = blockSize;
        end
        l = l+1;
    end    
    
    % Code differential frames for original coder
    if i == 1 || mod(i,iFrameFreq) == 0                
        dpcmBlocks = blocks;
        anchorBlocks = blocks;
    else                
        dpcmBlocks = blocks - anchorBlocks;       
    end
    
    rateOrig = 0;
    % Original coding, transform and quantize
    for p = 1:length(dpcmBlocks)
        quantizedOrigBlocks(:,:,p) = quantizer(dpcmBlocks(:,:,p),totalq);
        % Entropy code quantized blocks
        minValOrig = abs(min(min(quantizedOrigBlocks(:,:,p))));
        [origCell, info] = norm2huff(uint8(quantizedOrigBlocks(:,:,p)+minValOrig));
        [tmpk,tmpl,huff] = find(info.huffcodes);
        [listOfBitCodes,symbols] = huffcodes2bin(info.huffcodes);        
        sortedCodes = zeros(1,length(huff));
        % Sort frequency of codes to match huffman bit codes (most frequent
        % => shortest bit codes)
        for j = 1:length(huff)
            sortedCodes(j) = sum(sum(quantizedOrigBlocks(:,:,p)==huff(j)));
        end        
        sortedCodes = sort(sortedCodes,'descend');
        bitsPerBlock = 0;            
        for j = 1:length(huff)                
            bitsPerBlock = bitsPerBlock + length(listOfBitCodes{j})*sortedCodes(j)+ceil(log2(huff(j))); % Temporary        
        end            
        if sum(origCell(:)) == 0 % Then all values in the Huffman coded block are equal and the Huffman coder will return zero, the value of the coefficients is then equal to minVal
            tempBlockOrig(:) = double(huff2norm(origCell,info)) + minValOrig;                     
            if sum(sum(quantizedOrigBlocks(:,:,p))) ~= 0
               bitsPerBlock =  bitsPerBlock + ceil(log2(quantizedOrigBlocks(1,1,p))); % Skifte til den verdien som er ulik 0
            else
               bitsPerBlock =  bitsPerBlock + 1;
            end
        else
            tempBlockOrig(:) = double(huff2norm(origCell,info)) - minValOrig;         
        end    
        rateOrig = rateOrig + bitsPerBlock;
        % Inverse quantization
        iQuantOrigBlocks(:,:,p) = iQuantizer(tempBlockOrig,totalq); 
        rdOrig(1,p) = bitsPerBlock;
    end     
    
    tempRecRSQBlocks = zeros(size(recRSQBlocks));
    rateRSQ = 0;
    % RSQ coding
    for p = 1:length(blocks)                        
        quantizedRSQBlocks(:,:,p) = quantizerNoDCT(blocks(:,:,p),totalq);   
%         Entropy code quantized blocks
        minVal = abs(min(min(quantizedRSQBlocks(:,:,p))));
        [rsqCell, info] = norm2huff(uint8(quantizedRSQBlocks(:,:,p)+minVal));        
        [tmpk,tmpl,huff] = find(info.huffcodes);
        [listOfBitCodes,symbols] = huffcodes2bin(info.huffcodes);
        bitsPerBlock = 0;    
        sortedCodes = zeros(1,length(huff));        
        % Sort frequency of codes to match huffman bit codes (most frequent
        % => shortest bit codes)
        for j = 1:length(huff)
            sortedCodes(j) = sum(sum(quantizedRSQBlocks(:,:,p)==huff(j)));
        end        
        sortedCodes = sort(sortedCodes,'descend');
        for j = 1:length(huff)                
            bitsPerBlock = bitsPerBlock + length(listOfBitCodes{j})*sortedCodes(j)+ceil(log2(huff(j)));        
        end                
        rateRSQ = rateRSQ + bitsPerBlock;
        tempBlock(:) = double(huff2norm(rsqCell,info)) - minVal;  
        if sum(rsqCell(:)) == 0 % Then all values in the Huffman coded block are equal and the Huffman coder does not work properly
            tempBlock(:) = double(huff2norm(rsqCell,info)) + minVal;          
        else
            tempBlock(:) = double(huff2norm(rsqCell,info)) - minVal;          
        end
        % Inverse quantization
        recRSQBlocks(:,:,p) = iQuantizerNoDCT(tempBlock,totalq);  
        rdRSQ(1,p) = bitsPerBlock;
    end         
            
    % Inverse DPCM
    if i == 1 || mod(i,iFrameFreq) == 0                                 
        recOrigBlocks = iQuantOrigBlocks; 
        recAnchorBlocks = iQuantOrigBlocks;
    else                 
        recOrigBlocks = iQuantOrigBlocks + recAnchorBlocks; 
    end
    
    % Reconstruct frames
    j = blockSize;
    k = blockSize;
    adaptiveRate = 0;
    for p = 1:length(recRSQBlocks)
        % RDO calculation original coder
        distortion = sum(sum(recOrigBlocks(:,:,p) - blocks(:,:,p))).^2;         
        rdOrig(2,p) = distortion;
        % RDO calculation RSQ coder
        distortion = sum(sum(recRSQBlocks(:,:,p) - blocks(:,:,p))).^2;         
        rdRSQ(2,p) = distortion;
        lagrangeRSQ = rdRSQ(2,p)+lambda.*rdRSQ(1,p);
        lagrangeOrig = rdOrig(2,p) + lambda.*rdOrig(1,p);
        % Insert the block with the lowest Lagrange value
        if lagrangeRSQ < lagrangeOrig
            recYFrames(k-(blockSize-1):k, j-(blockSize-1):j,i) = recRSQBlocks(:,:,p);
            adaptiveRate = adaptiveRate + rdRSQ(1,p);
        else
            recYFrames(k-(blockSize-1):k, j-(blockSize-1):j,i) = recOrigBlocks(:,:,p);
            adaptiveRate = adaptiveRate + rdOrig(1,p);
        end        
        recOrigYFrames(k-(blockSize-1):k, j-(blockSize-1):j,i) = recOrigBlocks(:,:,p);
        recRSQYFrames(k-(blockSize-1):k, j-(blockSize-1):j,i) = recRSQBlocks(:,:,p);
        k = k + blockSize;
        if mod(p,nHeight/blockSize) == 0
           j = j+blockSize;
           k = blockSize;
        end
    end
    
    origPSNR = psnrcalc(yFrames(:,:,i),recOrigYFrames(:,:,i));    
    origPSNRVector(i) = origPSNR;
    totalRateOrig = totalRateOrig + rateOrig;
    fprintf('Original coder PSNR: %d \n',origPSNR);
    fprintf('Original coder rate: %d \n',rateOrig);

    rsqPSNR = psnrcalc(yFrames(:,:,i),recRSQYFrames(:,:,i));    
    rsqPSNRVector(i) = rsqPSNR;
    totalRateRSQ = totalRateRSQ + rateRSQ;
    fprintf('RSQ coder PSNR: %d \n',rsqPSNR);
    fprintf('RSQ coder rate: %d \n',rateRSQ);

    adaptivePSNR = psnrcalc(yFrames(:,:,i),recYFrames(:,:,i));    
    psnrVector(i) = adaptivePSNR;
    totalRateAdaptive = totalRateAdaptive + adaptiveRate;
    fprintf('Adaptive coder PSNR: %d \n',adaptivePSNR);
    fprintf('Adaptive coder rate: %d \n',adaptiveRate);
    fprintf('New frame\n');    
end

% Create yuv file
for j = 1:nFrames    
    % Convert to cells for yuv exporting
    yCells(j) = mat2cell(recYFrames(:,:,j),nHeight,nWidth);
    rsqYCells(j) = mat2cell(recRSQYFrames(:,:,j),nHeight,nWidth);
    origYCells(j) = mat2cell(recOrigYFrames(:,:,j),nHeight,nWidth);
    uCells(j) = mat2cell(uFrames(:,:,j),nHeight,nWidth);
    vCells(j) = mat2cell(vFrames(:,:,j),nHeight,nWidth);
end

% Export to yuv file
yuv_export(yCells,uCells,vCells,'seq_test3.yuv',nFrames,'w');
yuv_export(rsqYCells,uCells,vCells,'seq_test3rsq.yuv',nFrames,'w');
yuv_export(origYCells,uCells,vCells,'seq_test3orig.yuv',nFrames,'w');