function [signal_q, delta]=quantize_rsq(signal,Nbit)
% Takes in the signal to be quantized and the amount
% of bits used for quantization. Returns the quantized
% signal along with delta (quantization step size).

% Quantize signal with Nbit using uniform quantization
% adapted to the signal dynamic range. 
% Operates on 1 - and 2-dimensional signals. 
% For 2D signals, it quantizes each row separately.

%Find delta
Min=min(min(signal));
Max=max(max(signal));
Nlevel=2^Nbit;
delta=(Max-Min)/Nlevel;

%Find representation levels
y=zeros(1,Nlevel);

%% Uncomment cell for regular uniform quantization
for i = 1:Nlevel
   y(i)=Min + delta*(2*i-1)/2;     
end

%% Uncomment cell for extended end-zone quantizer

% % Set representations levels based on delta
% for i = 2:Nlevel-1
%    y(i)=Min + delta*(2*i-1)/2;     
% end
% % The first and last representation levels are set to 
% % the signals minimum and maximum value respectively.
% y(1) = Min;
% y(Nlevel) = Max;


%%
[M,N]=size(signal);
signal_q=zeros(M,N);

% Iterate over signal to map signal values to 
% representation values.
for m=1:M
    for n=1:N        
        index=ceil((signal(m,n)-Min)/delta);
        if index==0 
            index=1;
        end
        signal_q(m,n)=round(y(index));
    end
end

