%% Custom Coder no transformation
clc
clear all

% Read avi file
movObj = mmreader('screen_content_1.avi');

% Convert to 4D matrix with coefficients
vidFrames = read(movObj);

display('RSQ coder, no prediction:')

% Find height, width and number of frames
sizeOfMatrix = size(vidFrames);
nHeight = sizeOfMatrix(1);
nWidth = sizeOfMatrix(2);
% nFrames = sizeOfMatrix(4);
nFrames = 10;

% Set quantization parameter, cannot be set to 0
totalq = 32;
      
% Initialize variables
rFrames = uint8(zeros(nHeight, nWidth,nFrames));
gFrames = uint8(zeros(nHeight, nWidth,nFrames));
bFrames = uint8(zeros(nHeight, nWidth,nFrames));

yFrames = zeros(nHeight, nWidth,nFrames);
uFrames = uint8(zeros(nHeight, nWidth,nFrames));
vFrames = uint8(zeros(nHeight, nWidth,nFrames));

dpcmYFrames = zeros(nHeight, nWidth,nFrames);  
recYFrames = zeros(nHeight, nWidth,nFrames);
codedYFrames = zeros(nHeight, nWidth,nFrames);
quantYFrames = zeros(nHeight, nWidth,nFrames);

yCells = cell(1,nFrames);
uCells = cell(1,nFrames);
vCells = cell(1,nFrames);

blockSize = 4;

% Create block functions for using with blockproc function
quant_fun = @(block_struct) quantizerNoDCT(block_struct.data, totalq);
iQuant_fun = @(block_struct) iQuantizerNoDCT(block_struct.data, totalq);

% Split into R, G and B frames
for i = 1:nFrames
    rFrames(:,:,i) = vidFrames(:,:,1,i);
    gFrames(:,:,i) = vidFrames(:,:,2,i);
    bFrames(:,:,i) = vidFrames(:,:,3,i);
    
    % Convert from RGB to YUV
    [yFrames(:,:,i), uFrames(:,:,i), vFrames(:,:,i)] = rgb2yuv(rFrames(:,:,i), gFrames(:,:,i), bFrames(:,:,i));    
end

% Encoding
for i = 1:nFrames        
    codedYFrames(:,:,i) = round(yFrames(:,:,i)/totalq);
end

infoVector = cell(5,nFrames);
huffmanCoded = cell(1,nFrames);
tempValue = zeros(1,nFrames);
tempBlock = zeros(nHeight,nWidth);
totalBits = 0;

% Entropy coding
for i = 1:nFrames
    % Huffman coder only works for uint8, so we add the lowest value to
    % ensure that all values are >= 0.
    tempValue(i) = abs(min(min(codedYFrames(:,:,i))));
    tempBlock = codedYFrames(:,:,i) + tempValue(i);
    % Entropy code frames
    [tempCell, info] = norm2huff(uint8(tempBlock(:)));
    [k,l,huff] = find(info.huffcodes);    
    % Create bitcodes from huffcodes
    listOfBitCodes = huffcodes2bin(info.huffcodes);         
    % Sort frequency of codes to match huffman bit codes (most frequent =
    % shortest bit codes)  
    sortedCodes = zeros(1,length(huff));
    for j = 1:length(huff)
        sortedCodes(j) = sum(sum(tempBlock==huff(j)));
    end     
    sortedCodes = sort(sortedCodes,'descend');        
    bitsPerFrame = 0;
    tmpBits = 0;
    % Find total bits used by multiplying the length of the used codes with
    % the frequency of the codeword, then add the translation table 
    for j = 1:length(huff)        
        tmpBits = length(listOfBitCodes{j})*sortedCodes(j);        
        bitsPerFrame = bitsPerFrame+tmpBits+ceil(log2(huff(j)));
    end    
    totalBits = totalBits + bitsPerFrame;
    infoVector(:,i) = struct2cell(info);
    huffmanCoded(i) = {tempCell};
end

% Display total number of bits
fprintf('The total number of bits used to code the sequence is %d\n', totalBits);

fields = fieldnames(info);
dim = 1; % Chosen such that size(infoVector(:,i),dim) == length(fields)

% Inverse entropy coding
for i = 1:nFrames 
    tempVector = cell2mat(huffmanCoded(i));                 
    info = cell2struct(infoVector(:,i),fields,dim);
    tempBlock(:) = huff2norm(tempVector,info);
    codedYFrames(:,:,i) = tempBlock - tempValue(i);    
end

% Decoding
for i = 1:nFrames       
    % Inverse quantization       
    recYFrames(:,:,i) = codedYFrames(:,:,i)*totalq;
end

% Create yuv file
for j = 1:nFrames    
    % Convert to cells for yuv exporting
    yCells(j) = mat2cell(recYFrames(:,:,j),nHeight,nWidth);
    uCells(j) = mat2cell(uFrames(:,:,j),nHeight,nWidth);
    vCells(j) = mat2cell(vFrames(:,:,j),nHeight,nWidth);
end

% Export to yuv file
yuv_export(yCells,uCells,vCells,'output_rsq_no_pred.yuv',nFrames,'w');

% Calculate PSNR
videoPSNR2 = zeros(1,nFrames);
for i = 1:nFrames
   videoPSNR2(i) = psnrcalc(yFrames(:,:,i),recYFrames(:,:,i)); 
end
display('The PSNR for each frame is:');
videoPSNR2