%% Custom Coder no transformation
clc
clear all

% Read avi file
movObj = mmreader('screen_content_1.avi');

% Convert to 4D matrix with coefficients
vidFrames = read(movObj);

display('RSQ coder, differential coding, extended end-zone quantizer:')

% Find height, width and number of frames
sizeOfMatrix = size(vidFrames);
nHeight = sizeOfMatrix(1);
nWidth = sizeOfMatrix(2);
nFrames = sizeOfMatrix(4);

% Set quantization parameter, cannot be set to 0
bits_for_quantizer = 4;
iFrameFreq = 8;
      
% Initialize variables
rFrames = uint8(zeros(nHeight, nWidth,nFrames));
gFrames = uint8(zeros(nHeight, nWidth,nFrames));
bFrames = uint8(zeros(nHeight, nWidth,nFrames));

yFrames = zeros(nHeight, nWidth,nFrames);
uFrames = uint8(zeros(nHeight, nWidth,nFrames));
vFrames = uint8(zeros(nHeight, nWidth,nFrames));

dpcmYFrames = zeros(nHeight, nWidth,nFrames);  
recYFrames = zeros(nHeight, nWidth,nFrames);
codedYFrames = zeros(nHeight, nWidth,nFrames);
quantYFrames = zeros(nHeight, nWidth,nFrames);

yCells = cell(1,nFrames);
uCells = cell(1,nFrames);
vCells = cell(1,nFrames);

blockSize = 4;

% Split into R, G and B frames
for i = 1:nFrames
    rFrames(:,:,i) = vidFrames(:,:,1,i);
    gFrames(:,:,i) = vidFrames(:,:,2,i);
    bFrames(:,:,i) = vidFrames(:,:,3,i);
    
    % Convert from RGB to YUV
    [yFrames(:,:,i), uFrames(:,:,i), vFrames(:,:,i)] = rgb2yuv(rFrames(:,:,i), gFrames(:,:,i), bFrames(:,:,i));    
end

% Encoding
for i = 1:nFrames         
    % DPCM
    quantYFrames(:,:,i) = yFrames(:,:,i);
    if i == 1 || mod(i,iFrameFreq) == 0        
        dpcmYFrames(:,:,i) = quantYFrames(:,:,i);
        tempFrame = quantYFrames(:,:,i);
    else                
        dpcmYFrames(:,:,i) = quantYFrames(:,:,i) - tempFrame;
    end    
    if sum(sum(dpcmYFrames(:,:,i))) ~= 0      
        codedYFrames(:,:,i) = quantize(dpcmYFrames(:,:,i),bits_for_quantizer);
    else
        codedYFrames(:,:,i) = 0; 
    end
end

minusInfoVector = cell(5,nFrames);
plusInfoVector = cell(5,nFrames);
minusHuffmanCoded = cell(1,nFrames);
plusHuffmanCoded = cell(1,nFrames);
minusIndicesVector = cell(1,nFrames);
plusIndicesVector = cell(1,nFrames);
tempValue = zeros(1,nFrames);
totalBits = 0;

% Entropy coding
for i = 1:nFrames
    % Huffman coder only works for uint8, so we add the lowest value to
    % ensure that all values are >= 0.    
    plusIndices = find(codedYFrames(:,:,i)>=0);
    minusIndices = find(codedYFrames(:,:,i)<0);
    tempFrame = codedYFrames(:,:,i);
    plusValues = tempFrame(plusIndices);
    minusValues = tempFrame(minusIndices);        
    bitsPerFrame = 0;    
    if isempty(minusValues) %sum(minusValues) ~= 0                
        minusInfoVector(:,i) = {0};
        minusHuffmanCoded(i) = {0};
    else  % only do this if there are negative values        
        tempValue(i) = abs(min(min(minusValues)));        
        tempMinus = minusValues + tempValue(i);
        [minusCell, minusInfo] = norm2huff(uint8(tempMinus(:)));        
        [k,l,minusHuff] = find(minusInfo.huffcodes);    
        [minusListOfBitCodes,minsymbols] = huffcodes2bin(minusInfo.huffcodes);      
        minusSortedCodes = zeros(1,length(minusHuff));
        for j = 1:length(minusHuff)
            minusSortedCodes(j) = sum(sum(tempMinus==minusHuff(j)));
        end     
        minusSortedCodes = sort(minusSortedCodes,'descend'); 
        for j = 1:length(minusHuff)        
            bitsPerFrame = bitsPerFrame + length(minusListOfBitCodes{j})*minusSortedCodes(j)+ceil(log2(minusHuff(j)));
        end
        minusInfoVector(:,i) = struct2cell(minusInfo);
        minusHuffmanCoded(i) = {minusCell};       
    end
    % Code frames    
    [plusCell, plusInfo] = norm2huff(uint8(plusValues(:)));
    % Create bitcodes from huffcodes
    [k1,l1,plusHuff] = find(plusInfo.huffcodes);   
    [plusListOfBitCodes,plussymbols] = huffcodes2bin(plusInfo.huffcodes);                     
    % Sort frequency of codes to match huffman bit codes (most frequent =
    % shortest bit codes)      
    plusSortedCodes = zeros(1,length(plusHuff));
    for j = 1:length(plusHuff)
        plusSortedCodes(j) = sum(sum(plusValues==plusHuff(j)));
    end     
    plusSortedCodes = sort(plusSortedCodes,'descend');    
    % Find total bits used by multiplying the length of the used codes with
    % the frequency of the codeword, then add the translation table 
        
    if sum(plusCell(:)) == 0 % Entire frame = 0 or one value
        for j = 1:length(plusHuff)        
            bitsPerFrame = bitsPerFrame + 1;
        end
        bitsPerFrame = bitsPerFrame + ceil(log2(255)); % assume max value
%         bitsPerFrame = bitsPerFrame + ceil(log2(plusValues(1)));
    else
        for j = 1:length(plusHuff)        
            bitsPerFrame = bitsPerFrame + length(plusListOfBitCodes{j})*plusSortedCodes(j)+ceil(log2(plusHuff(j)));
        end        
    end
    totalBits = totalBits + bitsPerFrame;    
    plusInfoVector(:,i) = struct2cell(plusInfo);    
    plusHuffmanCoded(i) = {plusCell};
    plusIndicesVector(:,i) = {plusIndices};
    minusIndicesVector(:,i) = {minusIndices};
end

% Display total number of bits
fprintf('The total number of bits used to code the sequence is %d\n', totalBits);

fields = fieldnames(plusInfo);
dim = 1; % Chosen such that size(infoVector(:,i),dim) == length(fields)

% Inverse entropy coding
for i = 1:nFrames     
    tempFrame2 = zeros(nHeight,nWidth);    
    minusValues = 0;    
    plusVector = cell2mat(plusHuffmanCoded(i));
    plusValues = double(huff2norm(uint8(plusVector(:)),cell2struct(plusInfoVector(:,i),fields,dim)));      
    tempFrame2(cell2mat(plusIndicesVector(:,i))) = plusValues(:);
    if ~isempty(cell2mat(minusIndicesVector(:,i)))            
        minusVector = cell2mat(minusHuffmanCoded(i));
        minusValues = double(huff2norm(uint8(minusVector(:)),cell2struct(minusInfoVector(:,i),fields,dim)));
        tempFrame2(cell2mat(minusIndicesVector(:,i))) = minusValues(:) - tempValue(i);
    end            
    codedYFrames(:,:,i) = tempFrame2;     
end

% Decoding
for i = 1:nFrames       
%     Inverse DPCM
    if i == 1 || mod(i,iFrameFreq) == 0                        
        recYFrames(:,:,i) = codedYFrames(:,:,i);
        tempRecFrame = codedYFrames(:,:,i);
    else                 
        recYFrames(:,:,i) = codedYFrames(:,:,i) + tempRecFrame;
    end       	
end

% Create yuv file
for j = 1:nFrames    
    % Convert to cells for yuv exporting
    yCells(j) = mat2cell(recYFrames(:,:,j),nHeight,nWidth);
    uCells(j) = mat2cell(uFrames(:,:,j),nHeight,nWidth);
    vCells(j) = mat2cell(vFrames(:,:,j),nHeight,nWidth);
end

% Export to yuv file
yuv_export(yCells,uCells,vCells,'rsq_dpcm_ee.yuv',nFrames,'w');

% Calculate PSNR
videoPSNR = zeros(1,nFrames);
for i = 1:nFrames
   videoPSNR(i) = psnrcalc(yFrames(:,:,i),recYFrames(:,:,i)); 
end
display('The PSNR for each frame is:');
videoPSNR